#!/usr/bin/perl -w
# 
# generate_cover_letters.pl generates a LaTeX file with the customized
# cover letter for each school and produces a PDF file.
# Modified from 2006 Matthew J. Miller by Pingle Wang (wangpingle.com)

use strict;
use File::Basename;
use lib dirname (__FILE__);
use Schools;

sub main;
sub generate_latex;

# Set this to 1 if you want to generate cover letters for all
# schools regardless of whether their DO_GENERATE_KEY is set.
my $generate_all_ = 1;

# The directory where the files will go (should be created already).
my $cover_letter_dir_ = "cover_letters";
# The prefix that will be prepended to each file.
my $fname_pre_ = "coverletter_";
# A log file to see an errors.
my $log_file_ = "coverletter.log";

main();

sub main {

	my $key;
	my $num_keys = scalar(keys(%Schools::univ_));
	my $ctr = 1;

	print "\n";

	if(-e $cover_letter_dir_."/".$log_file_) {
		unlink($cover_letter_dir_."/".$log_file_);
	}

	my $i;
	if($generate_all_ == 1) {
		my @prev_fnames = glob($cover_letter_dir_."/".$fname_pre_."*");
		foreach $i (@prev_fnames) {
			unlink $i;
		}
	}

	foreach $key (keys(%Schools::univ_)) {
		if($generate_all_ == 1 || 
				(defined $Schools::univ_{$key}{$Schools::DO_GENERATE_KEY} &&
				$Schools::univ_{$key}{$Schools::DO_GENERATE_KEY} > 0)) {
			print "Generating $key, $ctr of $num_keys\n";
			generate_latex($key);
			$ctr++;
		}
	}

	print "\n";

}

# NOTE: You should change the LaTeX in this function
# to generate the format you desire.  I used the newlfm
# and charter (a font) packages to create my files.

sub generate_latex {


	my $key = shift;
	my $fname = $cover_letter_dir_."/".$fname_pre_.$key.".tex";
	local *OUT;

	open(OUT, '>', $fname) || die "Cannot open $fname\n";

	# newlfm is available at:
	# http://www.tug.org/tex-archive/help/Catalogue/entries/newlfm.html
	print OUT "\\documentclass[10pt,stdletter,sigleft]{newlfm}\n";
	print OUT "\\usepackage{charter}\n\n";
	print OUT "\\usepackage{etoolbox}\n\\makeatletter\n\\patchcmd{\\\@zfancyhead}{\\fancy\@reset}{\\f\@nch\@reset}{}{}\n\\patchcmd{\\\@set\@em\@up}{\\f\@ncyolh}{\\f\@nch\@olh}{}{}\n\\patchcmd{\\\@set\@em\@up}{\\f\@ncyolh}{\\f\@nch\@olh}{}{}\n\\patchcmd{\\\@set\@em\@up}{\\f\@ncyorh}{\\f\@nch\@orh}{}{}\n\\makeatother\n";
	# Determines how forceful LaTeX is in making sure page breaks
	# do not leave orphaned lines.
	print OUT "\\widowpenalty=1000\n";
	print OUT "\\clubpenalty=1000\n\n";

	# Replace "figures/logo" with the logo you want to use and
	# ".eps" with the extension of your file.
	print OUT "\\newsavebox{\\Luiuc}\n";
	print OUT "\\sbox{\\Luiuc}{%\n";
	print OUT "\t\\parbox[b]{1.75in}{%\n";
	print OUT "\t\t\\vspace{0.3in}%\n";
	print OUT "\t\t\\includegraphics[width=0.5\\textwidth]\n";
	print OUT "\t\t{./figures/your_school_logo.png}%\n";
	print OUT "\t}%\n";
	print OUT "}%\n";
	print OUT "\\makeletterhead{Uiuc}{\\Lheader{\\usebox{\\Luiuc}}}\n\n";

	print OUT "\\newlfmP{headermarginskip=0pt}\n";
	print OUT "\\newlfmP{sigsize=50pt}\n";
	print OUT "\\newlfmP{dateskipbefore=-1.6cm}\n";
	print OUT "\\newlfmP{dateskipafter=20pt}\n";
	print OUT "\\newlfmP{addrtoskipbefore=-40pt}\n";
	print OUT "\\newlfmP{sigskipbefore=0pt}\n";
	print OUT "\\newlfmP{sigskipafter=-100pt}\n";
	print OUT "\\newlfmP{addrfromphone}\n";
	print OUT "\\newlfmP{addrfromemail}\n";
	print OUT "\\newlfmP{addrfromfax}\n";
	print OUT "\\PhrPhone{Phone}\n";
	print OUT "\\PhrFax{Website}\n";
	print OUT "\\PhrEmail{Email}\n\n";

	print OUT "\\lthUiuc\n\n";
	print OUT "\\dateset{}\n\n";
	print OUT "\\namefrom{First Last}\n";
	print OUT "\\addrfrom{%\n";
	print OUT "\tXXX Street\\\\\n";
	print OUT "\tUniversity of YYY\\\\\n";
	print OUT "\tCity, State zip\n";
	print OUT "}\n";
	print OUT "\\phonefrom{123-456-7890}\n";
	print OUT "\\emailfrom{youremail\@xyz.com}\n";
	print OUT "\\faxfrom{yourwebsite.com}\n\n";

	if(defined $Schools::univ_{$key}{$Schools::ADDR_KEY}) {
		#print OUT "\t\\today \\vspace{0.2cm}\\\\\n";
		print OUT "\\nameto{".
			"\tNovember 10, 2019 \\vspace{0.2cm}\\\\\nFinance Recruiting Committee}\n";
		print OUT "\\addrto{%\n";
		print OUT "$Schools::univ_{$key}{$Schools::ADDR_KEY}";
		print OUT "}\n\n";
	}

	# Specify the default string to use if no TO_NAME key is defined.
	my $to_str = "Dear Finance Recruiting Committee";
	#my $to_str = "Dear Faculty Search Committee";
	if(defined $Schools::univ_{$key}{$Schools::TO_NAME_KEY}) {
		my @name = split(/ /, 
			$Schools::univ_{$key}{$Schools::TO_NAME_KEY});
		# $to_str = "Dear $name[0] $name[$#name]";
		$to_str = "Dear Finance Recruiting Committee";
	}

	print OUT "\\greetto{\\vspace{-0.2cm}$to_str,}\n";
	print OUT "\\closeline{Sincerely,}\n\\signature{\\includegraphics[width=2.5cm]{./figures/your_signature.png}}\n";

	print OUT "\\begin{document}\n";
	print OUT "\\begin{newlfm}\n\n";

	# Specify a default string to use if no UNIV_KEY or DEPT_KEY is defined.
	my $dept_str = "your department";
	if(defined $Schools::univ_{$key}{$Schools::UNIV_KEY} && 
			defined $Schools::univ_{$key}{$Schools::DEPT_KEY}) {
		# Make the school possessive...deal with school names ending
		# in 's'.
		my $poss_str = "";
		my $name_str = $Schools::univ_{$key}{$Schools::UNIV_KEY};
		if(substr($name_str,0, 4) eq "Univ") {
			$poss_str = "the ";
		}
		$dept_str = "the $Schools::univ_{$key}{$Schools::DEPT_KEY} in $poss_str$Schools::univ_{$key}{$Schools::UNIV_KEY}";
	}

	my $transcript_str = "";
	if(defined $Schools::univ_{$key}{$Schools::TRANSCRIPT}) {
		# If the ad is relevant to your specific background, use this
		# string.
		$transcript_str = "transcript, ";
	}

	my $rs_str = "";
	if(defined $Schools::univ_{$key}{$Schools::RESEARCH}) {
		# If the ad is relevant to your specific background, use this
		# string.
		$rs_str = "a research statement, ";
	}

	# Here's the actual text that will go in your cover letter.
	# I will leave mine in as an example, but it is NOT a template.
	# DO NOT PLAGERIZE THIS!!!  The cover letter should be unique
	# to you.  Plus, it is a well-known fact that the job gods
	# look very unfavorably upon anyone that doesn't write their
	# own cover letter.

	print OUT "\\vspace{-0.2cm}\nI am writing to apply for the position of Assistant Professor of Finance at $dept_str. \n".
	"I am currently a doctoral candidate in finance at XXX and expect to defend my dissertation by YYY.\n";

	print OUT "My job market paper XXX  \n\n";

	print OUT "My other projects YYY and ZZZ\n\n";

	print OUT "During my time as a Ph.D. student at XXX, I have taught YYY \n\n";

	print OUT "I will appreciate the opportunity to discuss my application and learn more about your institution at the coming CONFERENCE. I have enclosed my CV, samples of academic research, $transcript_str $rs_str and a teaching statement. Letters of reference will arrive shortly. Thank you for your consideration and I am happy to provide any additional information. \n\n";
	print OUT "\\end{newlfm}\n";
	print OUT "\\end{document}\n\n";

	close OUT;

	chdir($cover_letter_dir_);
	# Change this command if you generate LaTeX with a different
	# series of commands.
	system("pdflatex -pdfps $fname_pre_$key >> $log_file_ 2>&1");
	system("rm -f $fname_pre_$key.aux $fname_pre_$key.log");
	chdir("..");

}


